{% extends "joyous/joyous_base.html" %}
{% load wagtailcore_tags wagtailimages_tags %}

{% block content %}
{% with event=page.overrides %}
  <div class="content">
    <div class="event clearfix">
      {% if page.status %}
      <div class="event-status">
        {{ page.status_text }}
      </div>
      {% endif %}
      <h1 class="event-heading">{{ page.exception_title }}</h1>
      <div class="event-image">
        {% image event.image width-350 %}
      </div>
      {% include "joyous/includes/who_when_where.html" with event=page %}
      <div class="event-extra-info">
        {{ page.extra_information|richtext }}
      </div> 
      <div class="event-details">
        {{ event.details|richtext }}
      </div> 
    </div>
    <div class="event-overrides page-index">
      <a class="item" href="{% pageurl event %}">See also <div class="item-heading">other occurences</div> of this event.</a>
    </div>
  </div>
{% endwith %}
{% endblock %}
